const fs = require('fs');
const path = require("path");
const inquirer = require('inquirer');
const logger = require('../utils/logger');
const utils = require("./index");

let packageFile             = path.resolve(process.cwd(), "package.json");
let templateDir             = path.resolve(process.cwd(), "../seui-template");
let templatePackageFile     = path.resolve(process.cwd(), "../seui-template/package.json");
let iosVersionFile          = path.resolve(process.cwd(), "../seui-template/plugins/seui/framework/ios/seui/seuiVersion.m");
let iosPodspecFile          = path.resolve(process.cwd(), "../seui-template/plugins/seui/framework/ios/seui.podspec");
if (!fs.existsSync(templatePackageFile)) {
    logger.fatal(`error dir`);
}

let androidGradle = function(name, newValue) {
    let file = templateDir + '/platforms/android/seuiApp/build.gradle';
    if (!fs.existsSync(file)) {
        return "";
    }
    //
    let value = "";
    let result = fs.readFileSync(file, 'utf8');
    let reg = new RegExp(`${name}\\s*=\\s*("*|'*)(.+?)\\1\\r*\\n`);
    let match = result.match(reg);
    if (utils.count(match) > 2) {
        value = match[2].trim();
        if (typeof newValue !== "undefined") {
            let newResult = result.replace(new RegExp(match[0], "g"), `${name} = ${match[1]}${newValue}${match[1]}\n`);
            fs.writeFileSync(file, newResult, 'utf8');
            value = newValue;
        }
    }
    return value;
};

inquirer.prompt([{
    type: 'input',
    name: 'seuiVersionCode',
    message: "请输入版本号",
    default: function () {
        return androidGradle("seuiVersionCode") || 1;
    },
    validate: function (value) {
        if (Math.ceil(value) === Math.floor(value) && Math.ceil(value) > 0) {
            return true;
        }
        return '输入格式错误，版本号应为整数。';
    }
}, {
    type: 'input',
    name: 'seuiVersionName',
    message: "请输入版本名称",
    default: function () {
        return androidGradle("seuiVersionName") || "1.0.0";
    },
    validate: function (value) {
        return value !== ''
    }
}, {
    type: 'confirm',
    message: "是否要求脚手架版本与框架版本一致？",
    name: 'cliEqual',
}]).then(answers => {
    //修改seui-cli版本
    let newResult = fs.readFileSync(packageFile, 'utf8').replace(/"version":\s*"(.*?)"/, `"version": "${answers.seuiVersionName}"`);
    fs.writeFileSync(packageFile, newResult, 'utf8');
    //修改seui-template版本
    newResult = fs.readFileSync(templatePackageFile, 'utf8').replace(/"version":\s*"(.*?)"/, `"version": "${answers.seuiVersionName}"`);
    if (answers.cliEqual === true) {
        newResult = newResult.replace(/"seuiclimin":\s*"(.*?)"/, `"seuiclimin": "${answers.seuiVersionName}"`);
    }
    fs.writeFileSync(templatePackageFile, newResult, 'utf8');
    //修改Android版本
    androidGradle("seuiVersionCode", answers.seuiVersionCode);
    androidGradle("seuiVersionName", answers.seuiVersionName);
    //修改iOS版本
    newResult = fs.readFileSync(iosVersionFile, 'utf8').replace(/return @"(.*?)";\s*\/\/versionCode/, `return @"${answers.seuiVersionCode}";    //versionCode`);
    newResult = newResult.replace(/return @"(.*?)";\s*\/\/versionName/, `return @"${answers.seuiVersionName}";    //versionName`);
    fs.writeFileSync(iosVersionFile, newResult, 'utf8');
    newResult = fs.readFileSync(iosPodspecFile, 'utf8').replace(/s.version(\s*)=(\s*)"(.*?)"/, `s.version$1=$2"${answers.seuiVersionName}"`);
    fs.writeFileSync(iosPodspecFile, newResult, 'utf8');
    //修改完成
    console.log();
    console.log("操作成功！");
}).catch(console.error);
